/*
 * gc global variable provides access to GUI Composer infrastructure components and project information.
 * For more information, please see the Working with Javascript guide in the online help.
 */
var gc = gc || {};
gc.services = gc.services || {};

var initComplete = false;
var templateObj;

var ds;                         // Stores energy trace debug server
var energyTrace;                // Stores energy trace instance
var etEnabled = false;          // Energy trace enabled flag
var LEA_Running = false;        // Signals LEA is running on the target device
var _guiRefresh = 4;            // GUI refresh rate (ms)
var _ticksD = 0;                // Tick counter for a timer (D)
var _ticksA = 0;                // Tick counter for a timer (A)
var timerD = null;              // Used to periodically upate the GUI eith energy trace data
var timerA = null;              // Used to track timeout of application on target device
var aveLEAPwr = 0;              // Keeps track of average power during LEA operation
var aveCPUPwr = 0;              // Keeps track of average power during CPU operation

var dlgProgDevice;
var initComplete = false;
var manifestUpdated = false;
var et2;
var connectBtn;
var connectBtnLabel = "CONNECT";

/*
*  Boilerplate code for creating computed data bindings
*/
document.addEventListener('gc-databind-ready', function() {});

/*
*  Boilerplate code for creating custom actions
*/
document.addEventListener('gc-nav-ready', function() {});

var backplaneEventHdlr = function(event){};

// Initialize energy trace
var initEnergyTrace = function(){
    if (templateObj){
        energyTrace = gc.services['ti-service-energytrace'];
		
        if(energyTrace) {
            ds = energyTrace.ds;
            if (ds){
                // Debug Server object acquired
            }
        }
    }
}

// Enable or disable energy trace
var enableEnergyTrace = function(flag) {
    etEnabled = flag;
    if(flag) {
        // enable
        initEnergyTrace();
        if(targetReady)
            templateObj.$.Play.disabled = false;
    }
    else {
        // disable
        if(templateObj.$.Play) {
            templateObj.$.Play.disabled = true;
        }
    }
}

// Start energy trace capture
var etCapture = function(duration) {
    
    // init GUI
    var durationSec = duration/1000;
    templateObj.$.Play.disabled = true;
    
    // start capture and timer to track timeout
    energyTrace.capture({duration: duration});
    startTimerD();
}

// Update data in the GUI
var updateData = function() {
    _ticksD++;
    var status = energyTrace.getStatus();
    
    if(status) {
        // update GUI
		if(LEA_Running)
		{
		    // Calculate average LEA power in uW
		    aveLEAPwr = ((status.curEnergy/status.curTime) * 1000).toFixed(2);
		    if(aveLEAPwr > 0)
		    {
		        // Update LEA portion of power graph
			    templateObj.$.POWER_GRAPH.series_0_values = aveCPUPwr + ';' + " " + aveLEAPwr +";";
			    templateObj.$.LEA_PWR_LABEL.label = aveLEAPwr;
		    }
		}
		else
		{
		    // Calculate average CPU power in uW
		    aveCPUPwr = ((status.curEnergy/status.curTime) * 1000).toFixed(2);
		    if(aveCPUPwr > 0)
		    {
		        // Update CPU portion of power graph
			    templateObj.$.POWER_GRAPH.series_0_values = aveCPUPwr + ';' + " " + aveLEAPwr +";";
			    templateObj.$.CPU_PWR_LABEL.label = aveCPUPwr;
		    }
		}
        
        // Check if capture ended during GUI update and react accordingly
        if(status.runState === 'halted') {
            captureDone();
        }
    }
    
    // force stop if capture runs longer than 10 sec, an error condition
    if(_ticksD*_guiRefresh >= 10000) {
        captureDone();
    }
}

// Start a timerD to periodically update GUI with ET data
var startTimerD = function() {
    if(!timerD) {
        _ticksD = 0;
        timerD = setInterval(updateData, _guiRefresh);
    }
}

// Stop timerD
var stopTimerD = function() {
    if(timerD) {
        clearInterval(timerD);
        timerD = null;
    }
}

var updateTimerA = function() {
    _ticksA++;
    
    // force stop if runs beyond capture period 20 sec, an error condition
    if(_ticksA*_guiRefresh >= 15000) {
        failure();
        stopTimerA();
        templateObj.$.Play.disabled = false;
    }
}

// Timeout Failure
var failure = function() {
    templateObj.$.CPU_PWR_LABEL.label = 'Error';
    templateObj.$.LEA_PWR_LABEL.label = 'Error';

    templateObj.$.CPU_LABEL.label = 'Error';
    templateObj.$.LEA_LABEL.label = 'Error';
}

// Start TimerA to periodically check for timeout
var startTimerA = function() {
    if(!timerA) {
        _ticksA = 0;
        timerA = setInterval(updateTimerA, _guiRefresh);
    }
}

// Stop timerA
var stopTimerA = function() {
    if(timerA) {
        clearInterval(timerA);
        timerA = null;
    }
}

//Stop enegery trace capture
var captureDone = function() {
    if(timerD) {
        stopTimerD();
    }
}

// Wait for DOMContentLoaded event before trying to access the application template
var init = function() {

    templateObj = document.querySelector('#template_obj');
        // Wait for the template to fire a dom-change event to indicate that it has been 'stamped'
        // before trying to access components in the application.
    	   
    gc.fileCache.readJsonFile('project.json').then(function(manifest){
        manifest.disableAutoConnect = true;
        return gc.fileCache.writeJsonFile('project.json',manifest);
    }).then(function(){
        manifestUpdated = true;
        
        var backplane = gc.services['ti-core-backplane'];
        backplane.addEventListener('btnVisibilityChanged',backplaneEventHdlr);
    });
    	
    	templateObj.addEventListener('dom-change',function(){
    	    if (initComplete) return;
    	    this.async(function(){
        	    initComplete = true;
        	    console.log("Application template has been stamped.");
      	        // Now that the template has been stamped, you can use 'automatic node finding' $ syntax to access widgets.
      	        // e.g. to access a widget with an id of 'widget_id' you can use templateObj.$.widgetId
				
				var splash = document.querySelector("#splash")
				if (splash) {
					splash.parentNode.removeChild(splash);
				}
				// Code to configure the icon in the browser tab: [...
				var link = document.createElement('link');
				link.type = 'image/png';
				link.rel = 'shortcut icon';
				link.href = 'images/ti_gui_icon.png';
				document.getElementsByTagName('head')[0].appendChild(link);
				// ..]
      	           
                var htmlImport = 'app/ti-service-energytrace2/ti-service-energytrace2.html';
                var elementName = "ti-service-energytrace2";
				var htmlImport2 = 'app/ti-widget-connect-button2/ti-widget-connect-button2.html';
				var elementName2 = "ti-widget-connect-button2";
				this.importHref(htmlImport2, function(e) {
                console.log("ti-widget-connect-button2 imported.")
				});
				
                this.importHref(htmlImport, function (e) {
                    
                     // Disable the play button
      	            templateObj.$.Play.disabled = true;
                    
                    // Create a new instance
                    et2 = document.createElement(elementName);
                    // And add it to the container
                    templateObj.$.viewport.appendChild(et2);
                    // Lets set a dummy property just to show how to do it
                    et2.id = "ti_service_energytrace";
            
                    if (et2){
                        et2.addEventListener('energyTraceReady',function(){
                                if(gc.services['ti-service-energytrace'])
                                    enableEnergyTrace(true);
                        });
                        et2.addEventListener('energyTraceDisabled',function(){
                             enableEnergyTrace(false);
                        });
                        et2.addEventListener('energyTraceCaptureDone',function(){
                            captureDone();
                        });
                    }
      	        
      	            // Event listener for drop-down
                    templateObj.$.Function_Droplist.addEventListener("selected", function(event){
                        var func = templateObj.$.Function_Droplist.selectedValue;
                        
                        // Switch based on the function chosen in the drop down
                        switch(func)
                        {
                            case "1": //msp_add_q15
                                templateObj.$.FUNC_DESC.label = "Addition of two real source vectors";
                                templateObj.$.Order_Input.disabled = true;
                                templateObj.$.Length_Input.visible = true;
                                templateObj.$.Length_Droplist.visible = false;
                                break;
                            case "2": //msp_mac_q15
                                templateObj.$.FUNC_DESC.label = "Multiply and accumulate of real source vectors";
                                templateObj.$.Order_Input.disabled = true;
                                templateObj.$.Length_Input.visible = true;
                                templateObj.$.Length_Droplist.visible = false;
                                break;
                            case "3": //msp_fft_fixed_q15
                                templateObj.$.FUNC_DESC.label = "Real forward FFT function with fixed scaling by two at each stage";
                                templateObj.$.Order_Input.disabled = true;
                                templateObj.$.Length_Input.visible = false;
                                templateObj.$.Length_Droplist.visible = true;
                                break;
                            case "4": //msp_complex_fft_fixed_q15
                                templateObj.$.FUNC_DESC.label = "Complex forward FFT function with fixed scaling by two at each stage";
                                templateObj.$.Order_Input.disabled = true;
                                templateObj.$.Length_Input.visible = false;
                                templateObj.$.Length_Droplist.visible = true;
                                break;
                            case "5": //msp_fir_q15
                                templateObj.$.FUNC_DESC.label = "Discrete-time convolution of a source vector with real coefficients to apply an FIR filter";
                                templateObj.$.Order_Input.disabled = false;
                                templateObj.$.Length_Input.visible = true;
                                templateObj.$.Length_Droplist.visible = false;
                                templateObj.$.Order_Input.maxValue = 256;
                                break;
                            case "6": //msp_biquad_cascade_df2_q15
                                templateObj.$.FUNC_DESC.label = "Cascaded direct form 2 biquad filter";
                                templateObj.$.Order_Input.disabled = false;
                                templateObj.$.Length_Input.visible = true;
                                templateObj.$.Length_Droplist.visible = false;
                                templateObj.$.Order_Input.maxValue = 32;
                                if(templateObj.$.Order_Input.value > 32)
                                {
                                    templateObj.$.Order_Input.value = 32;
                                }
                                break;
                            default:
                                // All cases covered above
                                break;
                        }
                    });
                    
        	        templateObj.$.Device_Dropdown.addEventListener('selected',function(event){
  	                    var deviceName = templateObj.$.Device_Dropdown.selectedText;
  	                    var outFileName = templateObj.$.Device_Dropdown.selectedValue;
  	                    var model = document.querySelector('ti-serialio-model');
  	                    model.deviceName = deviceName;
  	                    model.targetProgram = outFileName;
          	        });
					var backplane = gc.services['ti-core-backplane'];
					backplane.hideProgressBar();
					gc.connectionManager.addEventListener('status-changed', function(event) {
						var backplane = gc.services['ti-core-backplane'];
						console.log("connectionManager.status = " + gc.connectionManager.status);
						switch (gc.connectionManager.status) {
							case 'disconnected':
								if (backplane) {
									backplane.hideProgressBar();
									backplane.setIcon('ti-core-icons:nolink');
								}
								templateObj.$.Program_Button.disabled = true;
								templateObj.$.Play.disabled = true;
								break;
							case 'connected':
								templateObj.$.Program_Button.disabled = templateObj.$.Auto_Load_Checkbox.checked;
								if (backplane) {
									backplane.hideProgressBar();
									backplane.setIcon('ti-core-icons:link');

								}
								break;
							case 'connecting':
								templateObj.$.Program_Button.disabled = true;
								break;
							default:
								templateObj.$.Program_Button.disabled = true;
								break;
						}
					});
                    templateObj.$.Program_Button.addEventListener('click',function(event){
         	            var dlg = document.querySelector('ti-service-programloader-dialog');
          	            dlg.open();
                    });
                    templateObj.$.Auto_Load_Checkbox.addEventListener('changed',function(event){
                        var model = document.querySelector('ti-serialio-model');
                        model.autoLoadProgram = templateObj.$.Auto_Load_Checkbox.checked;
                        templateObj.$.Program_Button.disabled = templateObj.$.Auto_Load_Checkbox.checked;
                    });
                
                    // Event listener for length input
                    templateObj.$.Length_Input.addEventListener("changed", function(event){
                     if(templateObj.$.Length_Input.value%2 !== 0)
                        {
                          templateObj.$.Length_Input.value = templateObj.$.Length_Input.value-1;
                        }
                    });
    			
    			    // Event listener for order input
    		    	templateObj.$.Order_Input.addEventListener("changed", function(event){
                        if(templateObj.$.Order_Input.value%2 !== 0)
                        {
                            templateObj.$.Order_Input.value = templateObj.$.Order_Input.value-1;
                        }
                    });

                    // Event listener for opening of serial port
                    templateObj.$.my_usb_uart.$.serialIO.addEventListener("serialPortOpen",function(event){
                        var backplane = gc.services['ti-core-backplane'];
                        backplane.setSelectedDevice('MSP430FR5994').then(function(){
                            var device = backplane.selectedDevice;
                            //the following will raise the debugServerConfigured event which will start the energy trace running.
                            backplane.configureDebugServer('MSP430FR5994.ccxml',backplane.selectedDeviceCcxmlString,true,true);
                        })
                    });
                
                    // Event listener for play button
                    templateObj.$.Play.addEventListener("click",function(event){
                        var func = templateObj.$.Function_Droplist.selectedValue;
                        var length;
                        var order = templateObj.$.Order_Input.value;
                        
                        // Determine whether length input should come from text box or drop down
                        if(templateObj.$.Function_Droplist.selectedValue == 3 | templateObj.$.Function_Droplist.selectedValue == 4)
                        {
                            length = templateObj.$.Length_Droplist.selectedValue;
                        }
                        else
                        {
                            length = templateObj.$.Length_Input.value;
                        }
                    
                        // Reset variables before running demo
                        templateObj.$.CPU_PWR_LABEL.label = 0;
                        templateObj.$.LEA_PWR_LABEL.label = 0;
                        templateObj.$.CPU_LABEL.label = 0;
                        templateObj.$.LEA_LABEL.label = 0;
                        templateObj.$.CYCLE_IMPROV.visible = false;
                        templateObj.$.PWR_IMPROV.visible = false;
                        aveCPUPwr = 0;
                        aveLEAPwr = 0;
                        templateObj.$.Cycles_Graph.series_0_values = "0; 0;";
                        templateObj.$.POWER_GRAPH.series_0_values = "0; 0;";
                    
                        // Start timer to detect application timeout and send JSON string to target, starts the demo
                        startTimerA();
                        templateObj.$.my_usb_uart.$.serialIO.serialSend('{"FUNCTION":'+ func +', "LENGTH":'+ length +', "ORDER":'+ order + '}\n');
                    });
                }.bind(templateObj));

            }.bind(templateObj),1);
          });
};

templateObj = document.querySelector('#template_obj');

if (templateObj) {
    init();
} else {
    document.addEventListener('DOMContentLoaded',init.bind(this))
}
